﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using log4net;
using MissionPlanner.ArduPilot;
using MissionPlanner.Controls;
using MissionPlanner.GeoRef;
using MissionPlanner.Joystick;
using MissionPlanner.Log;
using MissionPlanner.Maps;
using MissionPlanner.Utilities;
using MissionPlanner.Warnings;
using System.IO;
using System.Windows.Forms;
using System.Reflection;
using System.Linq.Expressions;

// Description: This plugin create a debug log of the plugin envrionment for testting isses 

namespace PluginTest
{
    public class ButtonPlugin : MissionPlanner.Plugin.Plugin
    {
        public override string Name
        {
            get { return "Test plugin environment"; }
        }

        public override string Version
        {
            get { return "1.0"; }
        }

        public override string Author
        {
            get { return "EOSBandi"; }
        }

        public override bool Init()
        {
            return true;
        }

        public override bool Loaded()
        {

            List<String> DisabledPluginNames = new List<String>();
            string logFileName = Settings.GetUserDataDirectory() + "PluginTest.log";
            //open file for writing, overwrite if exists

            foreach (var s in Settings.Instance.GetList("DisabledPlugins"))
                DisabledPluginNames.Add(s);

            System.IO.StreamWriter file = new System.IO.StreamWriter(logFileName, false);
            file.WriteLine("**OS version:" + Environment.OSVersion.ToString());

            string vpath = Path.GetDirectoryName(Application.ExecutablePath);
            vpath = vpath + Path.DirectorySeparatorChar + "version.txt";
            file.WriteLine("**Version file:" + vpath);
            if (File.Exists(vpath)) file.WriteLine("Version:" + File.ReadAllText(vpath));
            else file.WriteLine("**Version file not found");

            file.WriteLine("**Mission Planner version:" + Application.ProductVersion);

            string path = Settings.GetRunningDirectory() + "plugins" +
              Path.DirectorySeparatorChar;

            file.WriteLine("**Plugin path: " + path);
            if (Directory.Exists(path))
            {
                file.WriteLine("Plugin files:");
                foreach (string f in Directory.GetFiles(path))
                {
                    file.WriteLine(f);
                }
            }
            else file.WriteLine("Plugin path not found");

            file.WriteLine("Loading of dll files in Mission Planner plugin directory:");
            String[] files = Directory.GetFiles(path, "*.dll");
            foreach (var s in files)
            {
                file.WriteLine("----------------------------------");

                file.WriteLine(Path.Combine(Environment.CurrentDirectory, s));

                if (!File.Exists(s) || !s.EndsWith(".dll", true, null) ||
                    s.ToLower().Contains("microsoft.") || s.ToLower().Contains("system.") ||
                    s.ToLower().Contains("missionplanner.grid.dll") || s.ToLower().Contains("usbserialforandroid"))
                { file.WriteLine("file " + s + " ignored based on filename filtering"); continue; }

                if (DisabledPluginNames.Contains(Path.GetFileName(s).ToLower()))
                {
                    file.WriteLine("file {0} is disabled in config.xml", Path.GetFileName(s));
                    continue;
                }

                if (File.Exists(s) && File.Exists(Path.GetDirectoryName(Application.ExecutablePath) +
                                     Path.DirectorySeparatorChar + Path.GetFileName(s)))
                {
                    file.WriteLine("file {0} is ignored because it is on the main folder as well", Path.GetFileName(s));
                }
                Assembly asm = null;
                try
                {
                    file.WriteLine("Trying to Load Assembly " + s);
                    asm = Assembly.LoadFile(s);
                    file.WriteLine("Assembly loaded" + s);
                    file.WriteLine("Assembly name: " + asm.FullName);
                    file.WriteLine("Assembly version: " + asm.GetName().Version.ToString());
                    file.WriteLine("Assembly location: " + asm.Location);
                    file.WriteLine("Assembly codebase: " + asm.CodeBase);
                    foreach (var t in asm.GetTypes())
                    {
                        file.WriteLine("Assembly type: " + t.ToString());
                    }

                }
                catch (Exception e)
                {
                    file.WriteLine("Assembly load failed");
                    file.WriteLine("Exception: " + e.ToString());
                    continue;
                }

            }
            file.WriteLine("----------------------------------");
            file.Close();
            return true;
        }

        public override bool Loop()
        {
            return true;
        }

        public override bool Exit()
        {
            return true;
        }
    }
}
